@@ -10,17 +10,49 @@ from functools import cache, cached_property
 import itertools
 import math
 import operator
+import random
 
 class BaseDie():
+    """ Abstract base die class for a die.
+    
+    A die is a discrete probability distribution with `int` weights.
+    The outcomes can be any hashable, comparable values.
+    
+    The weights should be strictly positive for public dice.
+    Some private calculations may use temporary dice with zero weights,
+    however, this is not recommended for public use.
+    
+    Subclasses implement the operations that make sense only for a particular number of dimensions.
+    """
+    
     # Abstract methods.
     
     @abstractmethod
     def unary_op(self, op, *args, **kwargs):
-        """ Returns a die representing the effect of performing the operation on the outcomes. """
+        """ Returns a die representing the effect of performing the operation on the outcomes.
+        
+        This is used for the operators `-, +, abs, ~, round, trunc, floor, ceil`.
+        """
     
     @abstractmethod
     def binary_op(self, other, op, *args, **kwargs):
-        """ Returns a die representing the effect of performing the operation on pairs of outcomes from the two dice. """
+        """ Returns a die representing the effect of performing the operation on pairs of outcomes from the two dice.
+        
+        The other operand is cast to a die (using `hdroller.die`) before performing the operation.
+        
+        This is used for the operators `+, -, /, //, %, **, <, <=, >=, >`.
+        
+        This is used for the logical operators `&, |, ^` on `bool()` of the outcome.
+        
+        Special operators:
+            * The `@` operator rolls the left die, then rolls the right die that many times and sums the outcomes.
+            * The `==` operator returns `True` iff the two dice have identical outcomes, weights, and `ndim`.
+                To get the chance of the two dice rolling the same outcome, use the `eq` method.
+            * The `!=` operator returns `True` iff the two dice do not have identical outcomes, weights, and `ndim`.
+                To get the chance of the two dice rolling the same outcome, use the `ne` method.
+        
+        The operators `<<, >>` are not implemented.
+        """
         
     # Construction.
 
@@ -33,6 +65,7 @@ class BaseDie():
         
         Args:
             data: A `FrozenSortedWeights` mapping outcomes to weights.
+            ndim: The number of dimensions of the outcomes.
         """
         self._data = data
         self._ndim = ndim
@@ -118,6 +151,9 @@ class BaseDie():
     def __neg__(self):
         return self.unary_op(operator.neg)
     
+    def __pos__(self):
+        return self.unary_op(operator.pos)
+    
     def __abs__(self):
         return self.unary_op(operator.abs)
     
@@ -145,12 +181,22 @@ class BaseDie():
     
     @staticmethod
     def _zero(x):
-        """ Creates a default instance of x's type. """
         return type(x)()
     
     def zero(self):
-        # TODO: Do we actually want to have mixed zeros?
-        return self.unary_op(self._zero).reduce()
+        """ Zeros all outcomes of this die. 
+        
+        This is done by calling the constructor for each outcome's type with no arguments.
+        
+        The result will have a single outcome with weight 1.
+        
+        Raises:
+            ValueError if the zeros did not resolve to a single outcome.
+        """
+        result = self.unary_op(BaseDie._zero)
+        if len(result) != 1:
+            raise ValueError('zero() did not resolve to a single outcome.')
+        return result.reduce()
     
     # Binary operators.
     
@@ -170,10 +216,30 @@ class BaseDie():
         other = hdroller.die(other)
         return other.binary_op(self, operator.sub)
     
+    def __mul__(self, other):
+        other = hdroller.die(other)
+        return self.binary_op(other, operator.mul)
+    
+    def __rmul__(self, other):
+        other = hdroller.die(other)
+        return other.binary_op(self, operator.mul)
+        
+    def __truediv__(self, other):
+        other = hdroller.die(other)
+        return self.binary_op(other, operator.truediv)
+    
+    def __rtruediv__(self, other):
+        other = hdroller.die(other)
+        return other.binary_op(self, operator.truediv)
+    
     def __floordiv__(self, other):
         other = hdroller.die(other)
         return self.binary_op(other, operator.floordiv)
     
+    def __rfloordiv__(self, other):
+        other = hdroller.die(other)
+        return other.binary_op(self, operator.floordiv)
+    
     def __pow__(self, other):
         other = hdroller.die(other)
         return self.binary_op(other, operator.pow)
@@ -186,6 +252,12 @@ class BaseDie():
         other = hdroller.die(other)
         return self.binary_op(other, operator.mod)
     
+    def __rmod__(self, other):
+        other = hdroller.die(other)
+        return other.binary_op(self, operator.mod)
+    
+    # Comparators.
+    
     def __lt__(self, other):
         other = hdroller.die(other)
         return self.binary_op(other, operator.lt)
@@ -218,6 +290,34 @@ class BaseDie():
         other = hdroller.die(other)
         return self.binary_op(other, operator.ne)
     
+    @staticmethod
+    def _sign(x):
+        z = BaseDie._zero(x)
+        if x > z:
+            return 1
+        elif x < z:
+            return -1
+        else:
+            return 0
+    
+    def sign(self):
+        """ Outcomes become 1 if greater than `zero()`, -1 if less than `zero()`, and 0 otherwise.
+        
+        Note that for `float`s, +0.0, -0.0, and nan all become 0.
+        """
+        return self.unary_op(BaseDie._sign)
+    
+    @staticmethod
+    def _cmp(x, y):
+        return BaseDie._sign(x - y)
+    
+    def cmp(self, other):
+        """ Returns a die with the possible outcomes 1, -1, and 0.
+        
+        These are if self rolls > other, < other, or neither respectively.
+        """
+        return self.binary_op(other, BaseDie._cmp)
+    
     # Logical operators.
     # These operate on bool(outcome).
     
@@ -241,7 +341,7 @@ class BaseDie():
         other = hdroller.die(other)
         return self.binary_op(other, self._or)
     
-    def __or__(self, other):
+    def __ror__(self, other):
         other = hdroller.die(other)
         return other.binary_op(self, self._or)
     
@@ -253,43 +353,20 @@ class BaseDie():
         other = hdroller.die(other)
         return self.binary_op(other, self._xor)
     
-    def __xor__(self, other):
+    def __rxor__(self, other):
         other = hdroller.die(other)
         return other.binary_op(self, self._xor)
     
     # Special operators.
     
-    def __mul__(self, other):
-        """ Roll the left die, then roll the right die that many times and sum. """
+    def __rmatmul__(self, other):
+        """ Roll the left die, then roll the right die that many times and sum the outcomes. 
+        
+        Note that this differs from d() in that an `int` on the right side is treated as a constant
+        rather than as a standard die.
+        """
         other = hdroller.die(other)
-        
-        subresults = []
-        subresult_weights = []
-        
-        max_abs_die_count = max(abs(self.min_outcome()), abs(self.max_outcome()))
-        for die_count, die_count_weight in self.items():
-            factor = other.total_weight() ** (max_abs_die_count - abs(die_count))
-            subresults.append(other.repeat_and_sum(die_count))
-            subresult_weights.append(die_count_weight * factor)
-        
-        subresults = _align(*subresults)
-        
-        data = defaultdict(int)
-        
-        for subresult, subresult_weight in zip(subresults, subresult_weights):
-            for outcome, weight in subresult.items():
-                data[outcome] += weight * subresult_weight
-            
-        return hdroller.die(data, ndim=other.ndim())
-    
-    def __rmul__(self, other):
-        other = hdroller.die(other)
-        return other.__mul__(self)
-    
-    def multiply(self, other):
-        """ Actually multiply the outcomes of the two dice. """
-        other = hdroller.die(other)
-        return self.binary_op(other, operator.mul)
+        return other @ self
     
     # Mixtures.
 
@@ -370,7 +447,6 @@ class BaseDie():
             if len(data) == 0:
                 return None
         else:
-            # TODO: simplify this
             total_reroll_weight = sum(weight for outcome, weight in self.items() if outcome in outcomes )
             rerollable_factor = total_reroll_weight ** max_depth
             stop_factor = self.total_weight() ** max_depth + total_reroll_weight ** max_depth
@@ -520,13 +596,9 @@ class BaseDie():
         return sum(self._data.values())
     
     def total_weight(self):
+        """ The total weight of all outcomes. """
         return self._total_weight
         
-    def median(self):
-        left_index = bisect.bisect_left(self.cweights(), self.total_weight() / 2)
-        right_index = bisect.bisect_right(self.cweights(), self.total_weight() / 2)
-        return (self.outcomes()[left_index] + self.outcomes()[right_index]) / 2
-        
     def mode(self):
         """ Returns a tuple containing the most common outcome(s) of the die.
         
@@ -535,6 +607,7 @@ class BaseDie():
         return tuple(outcome for outcome, weight in self.items() if weight == self.modal_weight())
     
     def modal_weight(self):
+        """ The highest weight of any single outcome. """
         return max(self.weights())
     
     def ks_stat(self, other):
@@ -554,6 +627,7 @@ class BaseDie():
         return tuple(weight / self.total_weight() for weight in self.weights())
     
     def pmf(self):
+        """ Probability mass function. The probability of rolling each outcome in order. """
         return self._pmf
     
     @cached_property
@@ -561,6 +635,7 @@ class BaseDie():
         return tuple(itertools.accumulate(self.weights()))
     
     def cweights(self):
+        """ Cumulative weights. The weight <= each outcome in order. """
         return self._cweights
     
     @cached_property
@@ -568,6 +643,7 @@ class BaseDie():
         return tuple(itertools.accumulate(self.weights()[:-1], operator.sub, initial=self.total_weight()))
     
     def sweights(self):
+        """ Survival weights. The weight >= each outcome in order. """
         return self._sweights
         
     @cached_property
@@ -575,6 +651,7 @@ class BaseDie():
         return tuple(weight / self.total_weight() for weight in self.cweights())
     
     def cdf(self):
+        """ Cumulative distribution function. The chance of rolling <= each outcome in order. """
         return self._cdf
         
     @cached_property
@@ -582,8 +659,31 @@ class BaseDie():
         return tuple(weight / self.total_weight() for weight in self.sweights())
     
     def sf(self):
+        """ Survival function. The chance of rolling >= each outcome in order. """
         return self._sf
     
+    # Rolling.
+    
+    def sample(self):
+        """ Returns a random roll of this die. 
+        
+        Do not use for security purposes.
+        """
+        # We don't use random.choices since that is based on floats rather than ints.
+        r = random.randrange(self.total_weight())
+        index = bisect.bisect_right(self.cweights(), r)
+        return self.outcomes()[index]
+    
+    # Type conversions out.
+    
+    def __bool__(self):
+        """ Dice are not considered to have truth values. 
+        
+        The result of a comparator operation is a die, which if considered to have a truth value,
+        would imply that dice are sortable.
+        """
+        raise TypeError('Dice do not have truth values.')
+    
     # Strings.
     
     def __str__(self):
@@ -595,7 +695,7 @@ class BaseDie():
         result =  '| ' + ' ' * (o-7) + f'Outcome | {weight_header} | Probability |\n'
         result += '|-' + '-' *  o +            ':|-' + '-' * w + ':|------------:|\n'
         for outcome, weight, p in zip(self.outcomes(), self.weights(), self.pmf()):
-            result += f'| {str(outcome):>{o}} | {weight:>{w}} | {p:11.3%} |\n'
+            result += f'| {str(outcome):>{o}} | {weight:>{w}} | {p:11.6%} |\n'
         return result
     
     def __repr__(self):
